/***************************************************************************
 *   Borsch interpreter                                                    *
 *   Copyright (C) 2006 Michal Turek                                       *
 *   http://woq.nipax.cz/                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; version 2 of the License   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "cnodejump.hpp"

namespace borsch
{

/////////////////////////////////////////////////////////////////////////////
//// Default constructor

CNodeJump::CNodeJump(LEXTOKEN type, const string& file, int line)
	:
	CNode(file, line),
	m_type(type)
{

}


/////////////////////////////////////////////////////////////////////////////
//// Destructor

CNodeJump::~CNodeJump(void)
{

}


/////////////////////////////////////////////////////////////////////////////
//// Execute()

CNodeValue CNodeJump::Execute(void)
{
	switch(m_type)
	{
		case LEX_BREAK:
			throw CBreakException(GetFile(), GetLine());

		case LEX_CONTINUE:
			throw CContinueException(GetFile(), GetLine());

		default:
			assert(false);
			return CNodeValue();
	}
}

/////////////////////////////////////////////////////////////////////////////
//// Dump()

void CNodeJump::Dump(ostream& os, int indent) const
{
	DumpIndent(os, indent);
	os << "<Jump type=\"" << GetTokenName(m_type) << "\"/>" << endl;
}


/////////////////////////////////////////////////////////////////////////////
//// operator<<

ostream& operator<<(ostream& os, const CNodeJump& node)
{
	node.Dump(os);
	return os;
}

}// namespace
